import bpy
import blf
import logging
from mathutils import Vector
from bpy_extras.view3d_utils import location_3d_to_region_2d
import math
from collections import OrderedDict

# 设置日志记录
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

def _(text_en, text_zh):
    """
    根据 Blender 的当前语言环境返回英文或中文字符串。
    如果语言以 'zh' 开头，则返回中文，否则返回英文。
    """
    lang = bpy.context.preferences.view.language
    if lang.startswith('zh'):
        return text_zh
    else:
        return text_en

class GridDimensionOverlay:
    overlays = {}
    
    def __init__(self, area):
        self.area = area
        self.handle = None
        
        # 为每个轴维护独立的 LRU 缓存
        self.last_positions_x = OrderedDict()
        self.last_positions_y = OrderedDict()
        self.last_positions_z = OrderedDict()
        
        # 初始化默认最大标注数量
        self.max_annotations_x = 200
        self.max_annotations_y = 200
        self.max_annotations_z = 200
        
        # 设置淡出开始数量
        self.fade_start_count_x = 180
        self.fade_start_count_y = 180
        self.fade_start_count_z = 180
        
        self.position_memory = 0.9
        self.color = (1.0, 1.0, 1.0)  # 默认统一颜色
        self.alpha = 1.0
        self.font_size = 12
        self.grid_scale = 1.0
        self.active = False
        
        self.fade_in_distance = 50.0
        self.fade_out_distance = 100.0
        
        self.cm_fade_spacing_start = 100.0
        self.cm_fade_spacing_end = 50.0
        
        self.edge_fade_distance = 50.0  # 视口边缘淡出距离（像素）

    def register(self, report_func):
        if self.handle is None:
            if self.area.as_pointer() not in GridDimensionOverlay.overlays:
                logger.info(f'Registering overlay for area: {self.area.as_pointer()}')
                window_region = None
                for region in self.area.regions:
                    if region.type == 'WINDOW':
                        window_region = region
                        break
                if window_region is None:
                    logger.error(f'WINDOW region not found for area: {self.area.as_pointer()}')
                    report_func({'ERROR'}, _("Failed to find WINDOW region, cannot register grid dimension.", "未找到WINDOW区域，无法注册网格标注。"))
                    return
                try:
                    self.handle = self.area.spaces[0].draw_handler_add(
                        self.draw_callback_px, (), 'WINDOW', 'POST_PIXEL'
                    )
                    GridDimensionOverlay.overlays[self.area.as_pointer()] = self
                    self.active = True
                    logger.info(f'Successfully registered overlay for area: {self.area.as_pointer()}')
                    self.area.tag_redraw()  # 强制区域重绘
                except Exception as e:
                    logger.exception("Failed to add draw handler.")
                    report_func({'ERROR'}, _("Error registering grid dimension: ", "注册网格标注时出错: ") + str(e))
            else:
                logger.warning(f'Overlay already registered for area: {self.area.as_pointer()}')
                report_func({'WARNING'}, _("Overlay already registered for this area.", "该区域已注册网格标注。"))

    def unregister(self):
        if self.handle is not None:
            try:
                bpy.types.SpaceView3D.draw_handler_remove(self.handle, 'WINDOW')
                logger.info(f'Unregistered overlay for area: {self.area.as_pointer()}')
                self.area.tag_redraw()  # 强制区域重绘
            except Exception as e:
                logger.exception("Failed to remove draw handler.")
            self.handle = None
            self.active = False
            GridDimensionOverlay.overlays.pop(self.area.as_pointer(), None)

    @staticmethod
    def is_overlay_active(area):
        active = area.as_pointer() in GridDimensionOverlay.overlays
        logger.debug(f'Checking if overlay is active for area: {area.as_pointer()}, Result: {active}')
        return active

    def update_settings(self, scene):
        self.color = scene.grid_dimension_color
        self.alpha = scene.grid_dimension_alpha
        self.font_size = scene.grid_dimension_font_size
        self.grid_scale = scene.unit_settings.scale_length
        self.fade_in_distance = scene.grid_dimension_fade_in_distance
        self.fade_out_distance = scene.grid_dimension_fade_out_distance
        self.cm_fade_spacing_start = scene.grid_dimension_cm_fade_spacing_start
        self.cm_fade_spacing_end = scene.grid_dimension_cm_fade_spacing_end
        self.edge_fade_distance = scene.grid_dimension_edge_fade_distance
        
        # 更新每个轴的最大标注数量
        self.max_annotations_x = scene.grid_dimension_max_annotations_x
        self.max_annotations_y = scene.grid_dimension_max_annotations_y
        self.max_annotations_z = scene.grid_dimension_max_annotations_z
        
        # 更新每个轴的淡出开始数量
        self.fade_start_count_x = max(0, self.max_annotations_x - 300)
        self.fade_start_count_y = max(0, self.max_annotations_y - 300)
        self.fade_start_count_z = max(0, self.max_annotations_z - 300)
        
        # 确保不超过各自的最大标注数量
        self.fade_start_count_x = min(self.fade_start_count_x, self.max_annotations_x)
        self.fade_start_count_y = min(self.fade_start_count_y, self.max_annotations_y)
        self.fade_start_count_z = min(self.fade_start_count_z, self.max_annotations_z)

    def get_view_orientation(self, rv3d):
        # 标准视图方向
        standard_views = {
            'FRONT': Vector((0, -1, 0)),
            'BACK': Vector((0, 1, 0)),
            'RIGHT': Vector((1, 0, 0)),
            'LEFT': Vector((-1, 0, 0)),
            'TOP': Vector((0, 0, 1)),
            'BOTTOM': Vector((0, 0, -1))
        }
        
        tolerance = math.radians(10)  # 10度容差
        view_rot = rv3d.view_rotation.normalized()
        view_dir = view_rot @ Vector((0, 0, -1))  # 视图方向
        
        max_dot = -1
        closest_view = 'OTHER'
        for view, direction in standard_views.items():
            dot = view_dir.normalized().dot(direction)
            if dot > max_dot:
                max_dot = dot
                closest_view = view
        
        if max_dot > math.cos(tolerance):
            return closest_view
        return 'OTHER'

    def draw_callback_px(self):
        # 检查当前区域是否为目标区域
        current_area = bpy.context.area
        if current_area is None or current_area.as_pointer() != self.area.as_pointer():
            return

        scene = bpy.context.scene
        self.update_settings(scene)

        # 获取WINDOW区域
        window_region = None
        for region in self.area.regions:
            if region.type == 'WINDOW':
                window_region = region
                break
        if window_region is None:
            logger.error(f'WINDOW region not found during draw for area: {self.area.as_pointer()}')
            return

        region = window_region
        space = self.area.spaces[0]
        rv3d = space.region_3d

        # 确定当前视图方向
        view_orientation = self.get_view_orientation(rv3d)
        logger.debug(f'Current view orientation: {view_orientation}')

        # 根据视图方向决定显示哪些轴
        axes_to_show = set(['X', 'Y', 'Z'])  # 默认显示所有轴
        if rv3d.view_perspective == 'ORTHO':
            if view_orientation in ['FRONT', 'BACK']:
                axes_to_show = set(['Z', 'X'])
            elif view_orientation in ['RIGHT', 'LEFT']:
                axes_to_show = set(['Z', 'Y'])
            elif view_orientation in ['TOP', 'BOTTOM']:
                axes_to_show = set(['X', 'Y'])
            else:
                # 用户正交视图，通过按键 5 切换
                axes_to_show = set(['X', 'Y', 'Z'])  # 显示所有轴
                if not scene.grid_dimension_show_z:
                    axes_to_show.discard('Z')
        elif rv3d.view_perspective == 'PERSP':
            # 在透视视图下，根据用户设置是否显示Z轴
            if not scene.grid_dimension_show_z:
                axes_to_show.discard('Z')

        # 获取视距和相机位置
        view_distance = rv3d.view_distance
        camera_location = rv3d.view_matrix.inverted().translation
        max_distance = max(100, view_distance * 2)

        # 确定网格间隔
        if view_distance >= 1_000_000:
            grid_interval = 100_000
        elif view_distance >= 100_000:
            grid_interval = 10_000
        elif view_distance >= 10_000:
            grid_interval = 1_000
        elif view_distance >= 1_000:
            grid_interval = 100
        else:
            grid_interval = self.grid_scale

        sub_visibility_threshold = max_distance * 0.8
        sub_intervals = [0.2, 0.4, 0.6, 0.8]

        # 计算每个轴的最小和最大坐标
        min_x = math.floor((camera_location.x - max_distance) / grid_interval) * grid_interval
        max_x = math.ceil((camera_location.x + max_distance) / grid_interval) * grid_interval
        min_y = math.floor((camera_location.y - max_distance) / grid_interval) * grid_interval
        max_y = math.ceil((camera_location.y + max_distance) / grid_interval) * grid_interval
        min_z = math.floor((camera_location.z - max_distance) / grid_interval) * grid_interval
        max_z = math.ceil((camera_location.z + max_distance) / grid_interval) * grid_interval

        positions = {
            'X': [],
            'Y': [],
            'Z': []
        }

        # 预计算网格线
        grid_lines = []
        if 'X' in axes_to_show:
            grid_lines += [('X', current_x) for current_x in self.frange(min_x, max_x, grid_interval)]
        if 'Y' in axes_to_show:
            grid_lines += [('Y', current_y) for current_y in self.frange(min_y, max_y, grid_interval)]
        if 'Z' in axes_to_show:
            grid_lines += [('Z', current_z) for current_z in self.frange(min_z, max_z, grid_interval)]

        for axis, coord in grid_lines:
            if axis == 'X':
                pos = Vector((coord, 0, 0))
                max_ann = self.max_annotations_x
                fade_start = self.fade_start_count_x
                last_pos = self.last_positions_x
            elif axis == 'Y':
                pos = Vector((0, coord, 0))
                max_ann = self.max_annotations_y
                fade_start = self.fade_start_count_y
                last_pos = self.last_positions_y
            elif axis == 'Z':
                pos = Vector((0, 0, coord))
                max_ann = self.max_annotations_z
                fade_start = self.fade_start_count_z
                last_pos = self.last_positions_z
            else:
                continue  # 未知轴，跳过

            distance = (pos - camera_location).length
            if distance <= max_distance:
                positions[axis].append((pos, distance, False, axis))
                # 添加子标注
                if distance <= sub_visibility_threshold:
                    next_coord = coord + grid_interval
                    for ratio in sub_intervals:
                        sub_coord = coord + (next_coord - coord) * ratio
                        # 检查子标注是否超过轴的范围
                        if axis == 'X' and sub_coord >= max_x:
                            break
                        if axis == 'Y' and sub_coord >= max_y:
                            break
                        if axis == 'Z' and sub_coord >= max_z:
                            break
                        if axis == 'X':
                            sub_pos = Vector((sub_coord, 0, 0))
                        elif axis == 'Y':
                            sub_pos = Vector((0, sub_coord, 0))
                        elif axis == 'Z':
                            sub_pos = Vector((0, 0, sub_coord))
                        sub_distance = (sub_pos - camera_location).length
                        if sub_distance <= sub_visibility_threshold:
                            positions[axis].append((sub_pos, sub_distance, True, axis))

        # 对每个轴分别排序
        for axis in positions:
            positions[axis].sort(key=lambda x: x[1])  # 按距离排序

        # 计算 cm 标签的透明度 based on screen_spacing
        def calculate_cm_alpha(pos_start, pos_end):
            try:
                pos_m1_2d = location_3d_to_region_2d(region, rv3d, pos_start)
                pos_m2_2d = location_3d_to_region_2d(region, rv3d, pos_end)
                if pos_m1_2d and pos_m2_2d:
                    screen_distance = (pos_m2_2d - pos_m1_2d).length
                    if self.cm_fade_spacing_start > self.cm_fade_spacing_end:
                        if screen_distance >= self.cm_fade_spacing_start:
                            return 1.0
                        elif screen_distance <= self.cm_fade_spacing_end:
                            return 0.0
                        else:
                            return (screen_distance - self.cm_fade_spacing_end) / (self.cm_fade_spacing_start - self.cm_fade_spacing_end)
            except Exception as e:
                logger.exception(f"Error calculating cm alpha: {e}")
            return 1.0

        # 为 X、Y、Z 轴分别计算 cm 标签的透明度
        cm_alpha_sp = {}
        cm_alpha_sp['X'] = calculate_cm_alpha(Vector((0, 0, 0)), Vector((grid_interval, 0, 0)))
        cm_alpha_sp['Y'] = calculate_cm_alpha(Vector((0, 0, 0)), Vector((0, grid_interval, 0)))
        if 'Z' in axes_to_show:
            cm_alpha_sp['Z'] = calculate_cm_alpha(Vector((0, 0, 0)), Vector((0, 0, grid_interval)))
        else:
            cm_alpha_sp['Z'] = 1.0  # 如果不显示Z轴，则设为1.0

        # 计算 cm 标签的透明度 based on view distance
        if self.fade_out_distance > self.fade_in_distance:
            if self.fade_in_distance <= view_distance < self.fade_out_distance:
                cm_alpha_vd = 1 - (view_distance - self.fade_in_distance) / (self.fade_out_distance - self.fade_in_distance)
            elif view_distance < self.fade_in_distance:
                cm_alpha_vd = 1.0
            else:
                cm_alpha_vd = 0.0
        else:
            cm_alpha_vd = 1.0

        cm_alpha_final = min(cm_alpha_sp.values()) * cm_alpha_vd

        # 视口边缘淡出处理
        screen_width = region.width
        screen_height = region.height

        # 批量绘制文本
        blf_batch = []

        # 预获取视口边界，避免在循环中重复计算
        view_left = 0
        view_right = screen_width
        view_bottom = 0
        view_top = screen_height

        # 获取 m 标签淡出距离
        m_fade_start_distance = scene.grid_dimension_m_fade_start_distance
        m_fade_end_distance = scene.grid_dimension_m_fade_end_distance

        # 定义轴向颜色
        axis_colors = {
            'X': (1.0, 0.0, 0.0),  # 红色
            'Y': (0.0, 1.0, 0.0),  # 绿色
            'Z': (0.0, 0.0, 1.0)   # 蓝色
        }

        # 处理每个轴的标注
        for axis in ['X', 'Y', 'Z']:
            if axis not in axes_to_show:
                continue  # 如果当前视图不显示该轴，则跳过

            for index, (pos, distance, is_sub, axis_label) in enumerate(positions[axis]):
                if axis == 'X':
                    max_ann = self.max_annotations_x
                    fade_start = self.fade_start_count_x
                    last_pos = self.last_positions_x
                elif axis == 'Y':
                    max_ann = self.max_annotations_y
                    fade_start = self.fade_start_count_y
                    last_pos = self.last_positions_y
                elif axis == 'Z':
                    max_ann = self.max_annotations_z
                    fade_start = self.fade_start_count_z
                    last_pos = self.last_positions_z
                else:
                    continue  # 未知轴，跳过

                if index >= max_ann:
                    break  # 超过该轴的最大标注数量，停止处理

                try:
                    pos_2d = location_3d_to_region_2d(region, rv3d, pos)
                    if pos_2d is not None:
                        # 视野裁剪：仅绘制在视口范围内的标注
                        if not (view_left <= pos_2d.x <= view_right and view_bottom <= pos_2d.y <= view_top):
                            continue

                        distance_factor = distance / max_distance
                        value = pos.x if axis == 'X' else (pos.y if axis == 'Y' else pos.z)
                        text = self.format_distance(value, is_sub)

                        # 判断是否为10的倍数的米标注
                        is_major_m = False
                        if not is_sub:
                            # 假设单位为米
                            try:
                                numeric_value = float(text.replace('m', '').replace('cm', '').strip())
                                is_major_m = math.isclose(numeric_value % 10, 0, abs_tol=1e-6)
                            except ValueError:
                                is_major_m = False

                        # 如果是非10倍数的米标注，计算额外的alpha
                        if not is_sub and not is_major_m:
                            if view_distance < m_fade_start_distance:
                                m_alpha = 1.0
                            elif m_fade_start_distance <= view_distance < m_fade_end_distance:
                                m_alpha = 1.0 - (view_distance - m_fade_start_distance) / (m_fade_end_distance - m_fade_start_distance)
                            else:
                                m_alpha = 0.0
                        else:
                            m_alpha = 1.0  # 主要标注和子标注保持原有alpha

                        # 计算与边缘的最小距离
                        min_dist_to_edge = min(pos_2d.x, screen_width - pos_2d.x, pos_2d.y, screen_height - pos_2d.y)

                        # 计算边缘透明度
                        if self.edge_fade_distance > 0:
                            if min_dist_to_edge >= self.edge_fade_distance:
                                edge_alpha = 1.0
                            elif min_dist_to_edge <= 0:
                                edge_alpha = 0.0
                            else:
                                edge_alpha = min_dist_to_edge / self.edge_fade_distance
                        else:
                            edge_alpha = 1.0

                        # 计算数量接近最大值时的淡出和缩小
                        if max_ann > 0 and index >= fade_start:
                            excess = index - fade_start
                            fade_range = max_ann - fade_start
                            if fade_range > 0:
                                count_alpha = max(0.0, 1.0 - (excess / fade_range))
                                scale = max(0.5, 1.0 - (excess / fade_range) * 0.5)  # 缩小到50%
                            else:
                                count_alpha = 1.0
                                scale = 1.0
                        else:
                            count_alpha = 1.0
                            scale = 1.0

                        # 计算最终透明度和缩放
                        if is_sub:
                            # 调整cm标注的透明度和缩放
                            cm_alpha_modifier = 0.7  # cm标注额外的透明度比例
                            cm_scale_modifier = 0.8  # cm标注额外的缩放比例
                            alpha = self.alpha * cm_alpha_final * edge_alpha * count_alpha * cm_alpha_modifier
                            scale = scale * cm_scale_modifier
                        else:
                            alpha = self.alpha * edge_alpha * count_alpha * m_alpha
                            # m标注保持原有缩放比例

                        alpha = max(0.0, min(alpha, self.alpha))  # 确保在0-1之间
                        scale = max(0.5, min(scale, 1.0))  # 确保缩放在0.5到1.0之间

                        if alpha <= 0.0:
                            continue  # 不生成透明度为0或更低的标注

                        # 确定标注颜色
                        if scene.grid_dimension_auto_color:
                            axis_color = axis_colors.get(axis, self.color)  # 根据轴向获取颜色，默认为统一颜色
                        else:
                            axis_color = self.color

                        # 添加到批量绘制列表，包含颜色
                        blf_batch.append((pos_2d.x + 5, pos_2d.y + 5, text, alpha, scale, axis_color))
                except Exception as e:
                    logger.exception(f'Error processing axis {axis} at index {index}: {e}')
                    continue

        # 批量绘制文本
        for x, y, text, alpha, scale, color in blf_batch:
            try:
                self.draw_text(x, y, text, alpha, scale, color)
            except Exception as e:
                logger.exception(f'Error drawing text "{text}" at ({x}, {y}): {e}')

    def frange(self, start, stop, step):
        """生成范围内的浮点数序列，类似于 range()"""
        while start <= stop:
            yield start
            start += step

    def smooth_value(self, current, last, memory_factor):
        if last is None:
            return current
        return last * memory_factor + current * (1 - memory_factor)

    def get_stable_transform(self, pos_2d, text):
        key = f"{text}_{int(pos_2d.x/20)}_{int(pos_2d.y/20)}"
        if key in self.last_positions_x or key in self.last_positions_y or key in self.last_positions_z:
            try:
                if key in self.last_positions_x:
                    last_pos = self.last_positions_x.pop(key)
                elif key in self.last_positions_y:
                    last_pos = self.last_positions_y.pop(key)
                elif key in self.last_positions_z:
                    last_pos = self.last_positions_z.pop(key)
                pos_2d = Vector((
                    self.smooth_value(pos_2d.x, last_pos.x, self.position_memory),
                    self.smooth_value(pos_2d.y, last_pos.y, self.position_memory)
                ))
            except Exception as e:
                logger.exception(f"Error smoothing position for key: {key}, Error: {e}")
        # 根据轴类型决定存储在哪个缓存中
        # 需要根据上下文确定轴，这里假设通过文本判断（需改进）
        # 这里只是示例，实际应根据轴传递正确的轴信息
        axis = 'X' if 'x' in text.lower() else ('Y' if 'y' in text.lower() else 'Z')
        if axis == 'X':
            self.last_positions_x[key] = pos_2d.copy()
            if len(self.last_positions_x) > self.max_annotations_x:
                removed_key, removed_value = self.last_positions_x.popitem(last=False)
                logger.debug(f'Removed X axis cached position: {removed_key}')
        elif axis == 'Y':
            self.last_positions_y[key] = pos_2d.copy()
            if len(self.last_positions_y) > self.max_annotations_y:
                removed_key, removed_value = self.last_positions_y.popitem(last=False)
                logger.debug(f'Removed Y axis cached position: {removed_key}')
        elif axis == 'Z':
            self.last_positions_z[key] = pos_2d.copy()
            if len(self.last_positions_z) > self.max_annotations_z:
                removed_key, removed_value = self.last_positions_z.popitem(last=False)
                logger.debug(f'Removed Z axis cached position: {removed_key}')
        return pos_2d

    def get_unit_scale(self, text):
        try:
            if 'm' in text and 'cm' in text:
                meters = float(text.split('m')[0])
                cm_part = text.split('m')[1].strip().split('cm')[0]
                value = meters + float(cm_part) / 100
            elif 'm' in text:
                value = float(text.split('m')[0])
            else:
                value = float(text.split('cm')[0]) / 100
        except (ValueError, IndexError) as e:
            logger.warning(f"Error parsing text '{text}': {e}")
            return 1.0
        abs_value = abs(value)
        if abs_value >= 1.0:
            return 1.2
        elif abs_value >= 0.5:
            return 1.1
        elif abs_value >= 0.1:
            return 1.0
        else:
            return 0.9

    def format_distance(self, value, is_sub=False):
        sign = "-" if value < 0 else ""
        abs_value = abs(value)
        if is_sub:
            return f"{sign}{abs_value * 100:.0f}cm"
        if abs_value >= 1.0:
            meters = int(abs_value)
            centimeters = int((abs_value - meters) * 100)
            if centimeters > 0:
                return f"{sign}{meters}m {centimeters}cm"
            else:
                return f"{sign}{meters}m"
        else:
            return f"{sign}{abs_value * 100:.0f}cm"

    def draw_text(self, x, y, text, alpha, scale, color):
        try:
            unit_scale = self.get_unit_scale(text) * scale
            font_size = max(1, int(self.font_size * unit_scale))  # 确保字体大小至少为1
            blf.color(0, color[0], color[1], color[2], alpha)
            blf.size(0, font_size)
            blf.enable(0, blf.SHADOW)
            blf.shadow(0, 3, 0.0, 0.0, 0.0, alpha * 0.8)
            blf.position(0, x, y, 0)
            blf.draw(0, text)
            blf.disable(0, blf.SHADOW)
        except Exception as e:
            logger.exception(f'Error drawing text "{text}" at ({x}, {y}): {e}')

class VIEW3D_PT_grid_dimension(bpy.types.Panel):
    """UI面板：网格尺寸标注开关"""
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_label = _("Grid Dimension", "网格尺寸")
    bl_category = _("Grid Dimension", "网格尺寸")

    def draw(self, context):
        layout = self.layout
        layout.use_property_split = True
        layout.use_property_decorate = False

        row = layout.row()
        is_active = GridDimensionOverlay.is_overlay_active(context.area)

        row.operator(
            "view3d.grid_dimension_stop" if is_active else "view3d.grid_dimension_start", 
            text=_("Grid Dimension", "网格尺寸标注"), 
            depress=is_active,
            icon='RADIOBUT_ON' if is_active else 'RADIOBUT_OFF'
        )

        if is_active:
            box = layout.box()
            scene = context.scene
            box.prop(scene, "grid_dimension_font_size", text=_("Font Size", "字体大小"))
            box.prop(scene, "grid_dimension_alpha", text=_("Alpha", "透明度"))
            box.prop(scene, "grid_dimension_color", text=_("Color", "标注颜色"))
            box.prop(scene, "grid_dimension_fade_in_distance", text=_("Fade-in Distance (cm)", "cm淡入视距"))
            box.prop(scene, "grid_dimension_fade_out_distance", text=_("Fade-out Distance (cm)", "cm淡出视距"))
            box.prop(scene, "grid_dimension_cm_fade_spacing_start", text=_("CM Fade Start Spacing", "cm淡出起始间距"))
            box.prop(scene, "grid_dimension_cm_fade_spacing_end", text=_("CM Fade End Spacing", "cm淡出结束间距"))
            box.prop(scene, "grid_dimension_edge_fade_distance", text=_("Edge Fade Distance", "边缘淡出距离"))
            
            # 添加每个轴的最大标注数量设置
            box.prop(scene, "grid_dimension_max_annotations_x", text=_("Max Annotations X", "X轴最大标注数量"))
            box.prop(scene, "grid_dimension_max_annotations_y", text=_("Max Annotations Y", "Y轴最大标注数量"))
            box.prop(scene, "grid_dimension_max_annotations_z", text=_("Max Annotations Z", "Z轴最大标注数量"))

            # 新增m单位淡出距离属性
            box.prop(scene, "grid_dimension_m_fade_start_distance", text=_("M Fade Start Distance", "m淡出起始视距"))
            box.prop(scene, "grid_dimension_m_fade_end_distance", text=_("M Fade End Distance", "m淡出结束视距"))
            
            # 添加显示Z轴标注的选项
            box.prop(scene, "grid_dimension_show_z", text=_("Show Z Axis Dimensions", "显示Z轴标注"))
            
            # 新增自动轴向颜色选项
            box.prop(scene, "grid_dimension_auto_color", text=_("Auto Axis Color", "自动轴向颜色"))

            overlay = GridDimensionOverlay.overlays.get(context.area.as_pointer())
            if overlay:
                overlay.update_settings(scene)

class GridDimensionStartOperator(bpy.types.Operator):
    """启动网格尺寸标注"""
    bl_idname = "view3d.grid_dimension_start"
    bl_label = _("Start Grid Dimension", "启动网格尺寸标注")
    bl_options = {'REGISTER'}

    def execute(self, context):
        if context.area.type != 'VIEW_3D':
            self.report({'WARNING'}, _("Current area is not a 3D View.", "当前区域不是3D视图"))
            return {'CANCELLED'}
        overlay = GridDimensionOverlay(context.area)
        if not GridDimensionOverlay.is_overlay_active(context.area):
            overlay.register(self.report)
        return {'FINISHED'}

class GridDimensionStopOperator(bpy.types.Operator):
    """停止网格尺寸标注"""
    bl_idname = "view3d.grid_dimension_stop"
    bl_label = _("Stop Grid Dimension", "停止网格尺寸标注")
    bl_options = {'REGISTER'}

    def execute(self, context):
        if context.area.type != 'VIEW_3D':
            self.report({'WARNING'}, _("Current area is not a 3D View.", "当前区域不是3D视图"))
            return {'CANCELLED'}
        overlay = GridDimensionOverlay.overlays.get(context.area.as_pointer())
        if overlay:
            overlay.unregister()
        return {'FINISHED'}

def update_overlay(self, context):
    for overlay in GridDimensionOverlay.overlays.values():
        overlay.update_settings(context.scene)

def register():
    bpy.utils.register_class(VIEW3D_PT_grid_dimension)
    bpy.utils.register_class(GridDimensionStartOperator)
    bpy.utils.register_class(GridDimensionStopOperator)

    # 为场景添加自定义属性，用于存储和控制标注的设置
    bpy.types.Scene.grid_dimension_color = bpy.props.FloatVectorProperty(
        name=_("Color", "标注颜色"),
        subtype='COLOR',
        default=(1.0, 1.0, 1.0),
        min=0.0,
        max=1.0,
        update=update_overlay
    )
    bpy.types.Scene.grid_dimension_alpha = bpy.props.FloatProperty(
        name=_("Alpha", "透明度"),
        default=0.75,
        min=0.0,
        max=1.0,
        update=update_overlay
    )
    bpy.types.Scene.grid_dimension_font_size = bpy.props.IntProperty(
        name=_("Font Size", "字体大小"),
        default=12,
        min=1,
        update=update_overlay
    )
    bpy.types.Scene.grid_dimension_fade_in_distance = bpy.props.FloatProperty(
        name=_("Fade-in Distance (cm)", "cm淡入视距"),
        description=_("Fully visible when view distance is less than this value.", "视距小于此值时，cm标注完全显示"),
        default=2.0,
        min=0.0,
        update=update_overlay
    )
    bpy.types.Scene.grid_dimension_fade_out_distance = bpy.props.FloatProperty(
        name=_("Fade-out Distance (cm)", "cm淡出视距"),
        description=_("Fully hidden when view distance is greater than this value.", "视距大于此值时，cm标注完全隐藏"),
        default=4.0,
        min=0.0,
        update=update_overlay
    )
    bpy.types.Scene.grid_dimension_cm_fade_spacing_start = bpy.props.FloatProperty(
        name=_("CM Fade Start Spacing", "cm淡出起始间距"),
        description=_("CM labels fully visible when spacing is greater than this pixel value.", "m标签间距大于此像素值时，cm标注完全显示"),
        default=2.0,
        min=0.0,
        update=update_overlay
    )
    bpy.types.Scene.grid_dimension_cm_fade_spacing_end = bpy.props.FloatProperty(
        name=_("CM Fade End Spacing", "cm淡出结束间距"),
        description=_("CM labels fully hidden when spacing is less than this pixel value.", "m标签间距小于此像素值时，cm标注完全隐藏"),
        default=4.0,
        min=0.0,
        update=update_overlay
    )
    bpy.types.Scene.grid_dimension_edge_fade_distance = bpy.props.FloatProperty(
        name=_("Edge Fade Distance", "边缘淡出距离"),
        description=_("Start fading labels when they are closer than this pixel distance to any viewport edge.", "标注距离视口任意边缘小于此像素值时开始淡出"),
        default=400.0,
        min=0.0,
        update=update_overlay
    )
    
    # 添加每个轴的最大标注数量属性
    bpy.types.Scene.grid_dimension_max_annotations_x = bpy.props.IntProperty(
        name=_("Max Annotations X", "X轴最大标注数量"),
        description=_("Maximum number of annotations for the X axis. Exceeding this will cause fading and scaling.", "X轴的最大标注数量，超过后将淡出并缩小"),
        default=800,
        min=250,
        update=update_overlay
    )
    bpy.types.Scene.grid_dimension_max_annotations_y = bpy.props.IntProperty(
        name=_("Max Annotations Y", "Y轴最大标注数量"),
        description=_("Maximum number of annotations for the Y axis. Exceeding this will cause fading and scaling.", "Y轴的最大标注数量，超过后将淡出并缩小"),
        default=800,
        min=250,
        update=update_overlay
    )
    bpy.types.Scene.grid_dimension_max_annotations_z = bpy.props.IntProperty(
        name=_("Max Annotations Z", "Z轴最大标注数量"),
        description=_("Maximum number of annotations for the Z axis. Exceeding this will cause fading and scaling.", "Z轴的最大标注数量，超过后将淡出并缩小"),
        default=800,
        min=250,
        update=update_overlay
    )

    # 新增m单位淡出距离属性
    bpy.types.Scene.grid_dimension_m_fade_start_distance = bpy.props.FloatProperty(
        name=_("M Fade Start Distance", "m淡出起始视距"),
        description=_("Start fading non-10m M labels when view distance reaches this value.", "视距达到此值时，非10m的m标注开始淡出"),
        default=10.0,
        min=0.0,
        update=update_overlay
    )
    bpy.types.Scene.grid_dimension_m_fade_end_distance = bpy.props.FloatProperty(
        name=_("M Fade End Distance", "m淡出结束视距"),
        description=_("Fully fade out non-10m M labels when view distance reaches this value.", "视距达到此值时，非10m的m标注完全淡出"),
        default=20.0,
        min=0.0,
        update=update_overlay
    )
    
    # 添加显示Z轴标注的属性
    bpy.types.Scene.grid_dimension_show_z = bpy.props.BoolProperty(
        name=_("Show Z Axis Dimensions", "显示Z轴标注"),
        description=_("Enable to display Z axis grid dimensions in perspective view.", "启用后，在透视视图中显示Z轴的网格尺寸标注"),
        default=False,
        update=update_overlay
    )
    
    # 添加自动轴向颜色的属性
    bpy.types.Scene.grid_dimension_auto_color = bpy.props.BoolProperty(
        name=_("Auto Axis Color", "自动轴向颜色"),
        description=_("Enable to have annotation colors follow the axis line colors.", "启用后，标注颜色将跟随所在轴向线的颜色"),
        default=False,
        update=update_overlay
    )

def unregister():
    for overlay in list(GridDimensionOverlay.overlays.values()):
        overlay.unregister()

    bpy.utils.unregister_class(VIEW3D_PT_grid_dimension)
    bpy.utils.unregister_class(GridDimensionStartOperator)
    bpy.utils.unregister_class(GridDimensionStopOperator)

    # 删除自定义属性
    del bpy.types.Scene.grid_dimension_font_size
    del bpy.types.Scene.grid_dimension_alpha
    del bpy.types.Scene.grid_dimension_color
    del bpy.types.Scene.grid_dimension_fade_in_distance
    del bpy.types.Scene.grid_dimension_fade_out_distance
    del bpy.types.Scene.grid_dimension_cm_fade_spacing_start
    del bpy.types.Scene.grid_dimension_cm_fade_spacing_end
    del bpy.types.Scene.grid_dimension_edge_fade_distance
    
    # 删除每个轴的最大标注数量属性
    del bpy.types.Scene.grid_dimension_max_annotations_x
    del bpy.types.Scene.grid_dimension_max_annotations_y
    del bpy.types.Scene.grid_dimension_max_annotations_z

    # 删除新增的m单位淡出距离属性
    del bpy.types.Scene.grid_dimension_m_fade_start_distance
    del bpy.types.Scene.grid_dimension_m_fade_end_distance
    
    # 删除显示Z轴标注的属性
    del bpy.types.Scene.grid_dimension_show_z
    
    # 删除自动轴向颜色的属性
    del bpy.types.Scene.grid_dimension_auto_color

if __name__ == "__main__":
    register()
